/******************************************************************************
* 
* CAEN SpA - Front End Division
* Via Vetraia, 11 - 55049 - Viareggio ITALY
* +390594388398 - www.caen.it
*
***************************************************************************//**
* \note TERMS OF USE:
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation. This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. The user relies on the 
* software, documentation and results solely at his own risk.
*
* \file     CFE_DGTZ_lib.h
* \brief    CAEN Front End - Digitizer Library
* \author   Carlo Tintori (c.tintori@caen.it)
* \version  0.1
* \date     2008/11/03
*            
* \warning  This library is under development; the list of the functions 
*           is still incomplete
* \todo     Complete and order the register address map. 
            Complete the bit masks for the registers
* \todo     Add the support for the readout with interrupts
*
* This library provide functions, structures and definitions for the CAEN
* digitizer family
******************************************************************************/

#ifndef __CFE_DGTZ_SDK_H
#define __CFE_DGTZ_SDK_H

#include "../include/CAENcom.h"


/*###########################################################################*/
/*
** DEFINITIONS
*/
/*###########################################################################*/


/******************************************************************************
* Digitizer Registers Address Map 
******************************************************************************/
// VME INTERFACE
#define CFE_DGTZ_MultiEventBuffer            0x0000  ///< Multi Event Buffer (data readout space)
#define CFE_DGTZ_VMEControlReg               0xEF00  ///< VME Control Register BCast Address
#define CFE_DGTZ_BltEvNumReg                 0xEF1C  ///< Num. Events in Block Transfer Register BCast Address
#define CFE_DGTZ_SWClearReg                  0xEF28  ///< Software Data Clear Register BCast Address
#define CFE_DGTZ_SWResetReg                  0xEF24  ///< Software Board Reset Register BCast Address

// BROADCAST CHANNEL ADDRESSING
#define CFE_DGTZ_ChannelConfigReg            0x8000  ///< Channel Configuration Register BCast Address
#define CFE_DGTZ_BlockOrganizationReg        0x800C  ///< Block Organization Register BCast Address
#define CFE_DGTZ_CustomSizeReg               0x8020  ///< Custom Size Register BCast Address 
#define CFE_DGTZ_DPP1Reg                     0x8024  ///< DPP1 Param Register BCast Address
#define CFE_DGTZ_DPP2Reg                     0x8028  ///< DPP2 Param Register BCast Address
#define CFE_DGTZ_DPP3Reg                     0x802C  ///< DPP3 Param Register BCast Address
#define CFE_DGTZ_DPP4Reg                     0x8030  ///< DPP4 Param Register BCast Address
#define CFE_DGTZ_InputDCOffsetReg            0x8098  ///< Input DC offset adjust Register BCast Address
#define CFE_DGTZ_AcquisitionControlReg       0x8100  ///< Acquisition Contro Register BCast Address 
#define CFE_DGTZ_SoftTriggerReg              0x8108  ///< Software Trigger Register BCast Address 
#define CFE_DGTZ_TriggerSourceMaskReg        0x810C  ///< Trigger Source Mask Register BCast Address 
#define CFE_DGTZ_TriggerOutputMaskReg        0x8110  ///< Trigger Output Mask Register BCast Address 
#define CFE_DGTZ_PostTriggerReg              0x8114  ///< Post Trigger Register BCast Address 
#define CFE_DGTZ_ChannelEnableMaskReg        0x8120  ///< Channel Enable Mask Register BCast Address 
#define CFE_DGTZ_MotherBoardFWRevisionReg    0x8124  ///< MotherBoard FW Revision Register BCast Address 
#define CFE_DGTZ_SoftSyncReg                 0x813C  ///< Software Sync (reset time tag)
#define CFE_DGTZ_BoardInfoReg                0x8140  ///< Board Info Register BCast Address 
#define CFE_DGTZ_DACoutModeReg               0x8144  ///< Front Panel DAC (MON) Mode Register BCast Address
#define CFE_DGTZ_EventSizeReg                0x814C  ///< Event Size Register BCast Address 
#define CFE_DGTZ_AnalogMonitor               0x8150  ///< Analog Monitor Register Address 
#define CFE_DGTZ_DACPulserSettingsReg        0x8154  ///< Front Panel DAC Pulser Register BCast Address

// INDIVIDUAL CHANNEL ADDRESSING
#define CFE_DGTZ_InputDCOffsetReg_Ch(x)      (0x1098 | ((x)<<8)) ///< Input DC offset adjust Register Indiv. Address 
#define CFE_DGTZ_ChannelFWRevisionReg_Ch(x)  (0x108C | ((x)<<8)) ///< Channel FW Revision Register Indiv. Address 
#define CFE_DGTZ_DPP1Reg_Ch(x)               (0x1024 | ((x)<<8)) ///< DPP1 Param Register Indiv. Address
#define CFE_DGTZ_DPP2Reg_Ch(x)               (0x1028 | ((x)<<8)) ///< DPP2 Param Register Indiv. Address
#define CFE_DGTZ_DPP3Reg_Ch(x)               (0x102C | ((x)<<8)) ///< DPP3 Param Register Indiv. Address
#define CFE_DGTZ_DPP4Reg_Ch(x)               (0x1030 | ((x)<<8)) ///< DPP4 Param Register Indiv. Address

/******************************************************************************
* Control Register Bit Masks 
******************************************************************************/
#define CFE_DGTZ_CTRL_SAMPLE_MODE       0x00000001 ///< CTRL[0]     Sample Mode
#define CFE_DGTZ_CTRL_TRG_OVERLAP       0x00000002 ///< CTRL[1]     Enable Trigger Overlap
#define CFE_DGTZ_CTRL_MEM_TEST          0x00000008 ///< CTRL[3]     Enable Test Pattern
#define CFE_DGTZ_CTRL_SEQ_ACCESS        0x00000010 ///< CTRL[4]     Sequential/Random (0/1) Access. Must be 1.  
#define CFE_DGTZ_CTRL_INDIV_TRIGGER     0x00000100 ///< CTRL[8]     Enable Individual Trigger 
#define CFE_DGTZ_CTRL_INVERT_INPUT      0x00000200 ///< CTRL[9]     Invert input signal 
#define CFE_DGTZ_CTRL_DUAL_TRACE        0x00000800 ///< CTRL[11]    Enable Dual Trace 
#define CFE_DGTZ_CTRL_TRACE1_SEL        0x00003000 ///< CTRL[13:12] Virtual Probe 1   
#define CFE_DGTZ_CTRL_TRACE2_SEL        0x0000C000 ///< CTRL[15:14] Virtual Probe 2     
#define CFE_DGTZ_CTRL_EN_SCOPE          0x00010000 ///< CTRL[16]    Enable Readout of waveforms
#define CFE_DGTZ_CTRL_EN_PEAK           0x00020000 ///< CTRL[17]    Enable Readout of energies (peaks)
#define CFE_DGTZ_CTRL_EN_TIME           0x00040000 ///< CTRL[18]    Enable Readout of time tags
#define CFE_DGTZ_CTRL_DIGMON_SEL        0x00700000 ///< CTRL[22:20] Digital Monitor (Virtual Probe 3)   


/******************************************************************************
* Mask definitions for the header data
******************************************************************************/
#define CFE_DGTZ_FILLER           0xFFFFFFFF                   ///< Datum is a filler
#define CFE_DGTZ_GLOB_HEADER      0xA                          ///< Global Header Marker
#define CFE_DGTZ_BOARD_ID(r)      (((r)>>23) & 0x1F)           ///< Board ID (GEO address)
#define CFE_DGTZ_NUM_WORD(r)      (((r)      & 0xFFFFFFF))     ///< Number of event words
#define CFE_DGTZ_CH_MASK(r)       (((r)      & 0xFF))          ///< Partecipating Channel Mask
#define CFE_DGTZ_PATTERN_MASK(r)  (((r)>>8)  & 0xFFFF)         ///< Pattern Mask
#define CFE_DGTZ_EV_NUM(r)        (((r)      & 0xFFFFFFFF))    ///< Event Number
#define CFE_DGTZ_DATA_TYPE(r)     (((r)>>28) & 0xF)            ///< Data Type


/*###########################################################################*/
/*
** REGISTER ACCESS LAYER 
*/
/*###########################################################################*/

/**************************************************************************//**
* \fn      int CFE_DGTZ_ResetDevice(int *dev)
* \brief   Reset the device (SW command)
*            
* \param   [IN]  dev: devive handler
* \return  0: success; negative numbers are error codes
******************************************************************************/
int CFE_DGTZ_ResetDevice(int dev);

/**************************************************************************//**
* \fn      int CFE_DGTZ_ClearData(int *dev)
* \brief   Clear the data of the acquisition (memory buffer)
*            
* \param   [IN]  dev: devive handler
* \return  0: success; negative numbers are error codes
******************************************************************************/
int CFE_DGTZ_ClearData(int dev);

/**************************************************************************//**
* \fn      int CFE_DGTZ_StartAcquisition(int *dev)
* \brief   Start the acquisition
*            
* \param   [IN]  dev: devive handler
* \return  0: success; negative numbers are error codes
******************************************************************************/
int CFE_DGTZ_StartAcquisition(int dev);

/**************************************************************************//**
* \fn      int CFE_DGTZ_StopAcquisition(int *dev)
* \brief   Stop the acquisition
*            
* \param   [IN]  dev: devive handler
* \return  0: success; negative numbers are error codes
******************************************************************************/
int CFE_DGTZ_StopAcquisition(int dev);

/**************************************************************************//**
* \fn      int CFE_DGTZ_SendSWtrig(int *dev)
* \brief   Send a SW trigger
*            
* \param   [IN]  dev: devive handler
* \return  0: success; negative numbers are error codes
******************************************************************************/
int CFE_DGTZ_SendSWtrig(int dev);

/**************************************************************************//**
* \fn      int CFE_DGTZ_SendSWsync(int *dev)
* \brief   Send a SW sync (reset time tag)
*            
* \param   [IN]  dev: devive handler
* \return  0: success; negative numbers are error codes
******************************************************************************/
int CFE_DGTZ_SendSWsync(int dev);


/*###########################################################################*/
/*
** DATA READOUT
*/
/*###########################################################################*/

/**************************************************************************//**
* \fn      int CFE_DGTZ_ReadBlock(int dev, int BltSize, uint32_t *buff, int *nw)
* \brief   Read a block of data from the device
*            
* \param   [IN]  dev: device handler
* \param   [IN]  MaxBlockSize: size of the Block Read Cycle (in bytes)
* \param   [OUT] buff: pointer to the data buffer
* \param   [OUT] nw: number of words read from the device 
* \return  0 = Success; negative numbers are error codes
******************************************************************************/
int CFE_DGTZ_ReadBlock(int dev, int MaxBlockSize, uint32_t *buff, int *nw);

/**************************************************************************//**
* \fn      int CFE_DGTZ_DPPGetChannelData(uint32_t *buffer, int ch, unit32_t ConfigReg, short *outwave[4], uint16_t *Elist, uint32_t *Tlist, int *np, int *nw)
* \brief   Retrieve the waveforms and/or the pulse energy/timetag of one channel from the event data
*            
* \param   [IN]  buffer: pointer to the first word of the event data
* \param   [IN]  ch: channel
* \param   [IN]  ConfigReg: content of the digitizer Channel Configuration Register
* \param   [OUT] outwave: pointers to the buffers for the waveform
* \param   [OUT] Elist: pointers to the buffers for the energy list
* \param   [OUT] Tlist: pointers to the buffers for the timetag list
* \param   [OUT] np: number of triggers (energy/timetag) found
* \return  Number of event words belonging to that channel
******************************************************************************/
int CFE_DGTZ_DPPGetChannelData(uint32_t *buffer, int ch, uint32_t ConfigReg, int16_t *outwave[4], uint16_t *Elist, uint32_t *Tlist, int *np);

#endif