/******************************************************************************
* 
* CAEN SpA - Front End Division
* Via Vetraia, 11 - 55049 - Viareggio ITALY
* +390594388398 - www.caen.it
*
***************************************************************************//**
* \note TERMS OF USE:
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation. This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. The user relies on the 
* software, documentation and results solely at his own risk.
*
* \file     CFE_LINK.h
* \brief    CAEN Front End - Link for the access to the hardware devices
* \author   Carlo Tintori (c.tintori@caen.it)
* \version  0.1
* \date     2008/11/03
*
* This library provides the basic I/O functions for the access to the hardware.
* CAEN Catalog products (V1718, V2718, A2818) are already supported (the 
* functions are actually a simple one to one wrapping to the CAENVMElib). If 
* you use a different VME bridge or CPU, you must provide the relevant 
* functions for the VME access.
******************************************************************************/

#ifndef __CFE_LINK_H
#define __CFE_LINK_H

#define CAENLINK  ///< remove this line if you don't use a CAEN bridge or optical link

/******************************************************************************
* Link Types
******************************************************************************/
#define CFE_LINK_V1718  0   ///< USB => VME => Device
#define CFE_LINK_V2718  1   ///< PCI => A2818 => CONET => V2718 => VME => Device
#define CFE_LINK_CONET  2   ///< PCI => A2818 => CONET => Device
#define CFE_LINK_USB    3   ///< USB => Device


#ifdef CAENLINK

#include "CAENVMElib.h"

// Wrapper to the CAENVMElib
#define CFE_LinkOpen(lt, ln, dn, h)             CAENVME_Init(lt, ln, dn, h)
#define CFE_LinkClose(h)                        CAENVME_End(h)
#define CFE_LinkWrite32(h, a, d)                CAENVME_WriteCycle(h, a, d, cvA32_U_DATA, cvD32)
#define CFE_LinkRead32(h, a, d)                 CAENVME_ReadCycle(h, a, d, cvA32_U_DATA, cvD32)
#define CFE_LinkReadBlock(h, a, d, s, n)        CAENVME_FIFOBLTReadCycle(h, a, d, s, cvA32_U_MBLT, cvD64, n);

#else

int CFE_LinkOpen(int LinkType, int LinkNumber, int DeviceNumber, int32_t *handle);
int CFE_LinkClose(int32_t handle);
int CFE_LinkWrite32(long handle, uint32_t address, uint32_t data); 
int CFE_LinkRead32(long handle, uint32_t address, uint32_t *data);                 
int CFE_LinkReadBlock(long handle, uint32_t address, uint32_t *data, unit32_t size, uint32_t *nb);  

#endif

#endif