/******************************************************************************
* 
* CAEN SpA - Front End Division
* Via Vetraia, 11 - 55049 - Viareggio ITALY
* +390594388398 - www.caen.it
*
***************************************************************************//**
* \note TERMS OF USE:
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation. This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. The user relies on the 
* software, documentation and results solely at his own risk.
*
* \file     DPPalg.h
* \brief    CAEN Front End - Digital Pulse Processing - OffLine algorithms
* \author   Carlo Tintori (c.tintori@caen.it)
* \version  0.1
* \date     2008/11/03
*
* This library provides the functions for the off-line DPP and for the 
* wavefor emulator.
******************************************************************************/

#ifndef __DPPALG_H
#define __DPPALG_H


/*##########################################################################*/
/*
** DPP algorithms
*/
/*##########################################################################*/

/**************************************************************************//**
* \fn      void DPPalgInit(DPPparams DPP)
* \brief   Initialize the DPP algorithms
*           
* \param   [IN]  DPP: struct with the DPP parameters
* \return  0: success, -1: error
******************************************************************************/
int DPPalgInit(DPPparams *DPP);

/**************************************************************************//**
* \fn      int DPPalg(unsigned short *wave, ACQparams *pACQ, int *np);
* \brief   Perform the DPP on the sampled signal contained in 'wave'
*           
* \param   [IN]  inwave: pointer to the input waveform data buffer (stream of samples)
* \param   [IN]  ns: number of samples in 'inwave'
* \param   [OUT] outwave: pointer to the output waveforms (virtual probes)
* \param   [IN]  vp: Virtual Probe Selection 
* \param   [OUT] Elist: pointers to the buffers for the energy list
* \param   [OUT] Tlist: pointers to the buffers for the timetag list
* \param   [OUT] np: number of triggers (energy/timetag) found
* \return  0: success, -1: error
******************************************************************************/
int DPPalg(uint16_t *inwave, int ns, uint16_t *outwave[4], int vp[3], int16_t *Elist, int32_t *Tlist, int *np);


/*##########################################################################*/
/*
** Waveform Emulator
*/
/*##########################################################################*/

/**************************************************************************//**
* \fn      void WEinit(WEparams WE);
* \brief   Initialize the waveform emulator
*            
* \param   [IN]  WE: struct with the parameters of the emulator
* \return  0: success, -1: error
******************************************************************************/
int WEinit(WEparams *WE);

/**************************************************************************//**
* \fn      void WEend();
* \brief   Close the waveform emulator
******************************************************************************/
void WEend();

/**************************************************************************//**
* \fn      void WaveEmulate(unsigned short *wave, WEparams WE);
* \brief   Generate a waveform
*            
* \param   [OUT] wave: pointer to the waveform data buffer (stream of samples)
* \param   [IN]  WE: struct with the parameters of the emulator
* \return  0: success, -1: error
******************************************************************************/
int WaveEmulate(uint16_t *wave, WEparams *WE);

#endif