/******************************************************************************
* 
* CAEN SpA - Front End Division
* Via Vetraia, 11 - 55049 - Viareggio ITALY
* +390594388398 - www.caen.it
*
***************************************************************************//**
* \note TERMS OF USE:
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation. This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. The user relies on the 
* software, documentation and results solely at his own risk.
*
* \file     DPPconfig.h
* \brief    CAEN Front End - Digital Pulse Processing - Acquisition Engine
* \author   Carlo Tintori (c.tintori@caen.it)
* \version  0.2
* \date     2008/11/03
*
******************************************************************************/
#ifndef __DPPCONFIG_H
#define __DPPCONFIG_H

/*###########################################################################*/
/*
** DEFINITIONS
*/
/*###########################################################################*/

/******************************************************************************
* RunTime Commands (from the socket of the Control Panel or from the stdin)
******************************************************************************/
#define CMD_QUIT        0x01  ///< Quit the program
#define CMD_RESTART     0x02  ///< Reload config files and restart the acquisition
#define CMD_CLEAR       0x04  ///< Flush data 
#define CMD_INITPLOT    0x08  ///< Reload the plot settings
#define CMD_RUNTOOL     0x10  ///< Run external tool

/**************************************************************************//**
* \fn      void ClearHistograms(ACQparams *ACQ)
* \brief   Clear all the histograms
* \param   [IN]  ACQ: struct with the parameters for the acquisition
******************************************************************************/
void ClearHistograms(ACQparams *ACQ);

/**************************************************************************//**
* \fn      int ParseConfigFile(char *FileName, ACQparams *ACQ, DPPparams *DPP, WEparams *WE, DGTZparams *DGTZ)
* \brief   Read a configuration file and parse the commands
* \param   [IN]  FileName: name of the file to parse
* \param   [IN]  ACQ: struct with the parameters for the acquisition
* \param   [IN]  WE: struct with the parameters for the emulator
* \param   [IN]  DGTZ: struct with the parameters for the digitizers
* \return  0: success, -1: error
******************************************************************************/
int ParseConfigFile(char *FileName, ACQparams *ACQ, WEparams *WE, DGTZparams *DGTZ);

/**************************************************************************//**
* \fn      int InitAcquisition(ACQparams *ACQ, DPPparams *DPP, WEparams *WE, DGTZparams *DGTZ)
* \brief   Initialize the digitizers and the variables for the acquisition
* \param   [IN]  ACQ: struct with the parameters for the acquisition
* \param   [IN]  WE: struct with the parameters for the emulator
* \param   [IN]  DGTZ: struct with the parameters for the digitizers
* \return  0: success, -1: error
******************************************************************************/
int InitAcquisition(ACQparams *ACQ, WEparams *WE, DGTZparams *DGTZ);

/**************************************************************************//**
* \fn      int EndAcquisition(ACQparams *ACQ, DPPparams *DPP, WEparams *WE, DGTZparams *DGTZ)
* \brief   Free the memory buffers, close all files and handes, atc...
* \param   [IN]  ACQ: struct with the parameters for the acquisition
* \param   [IN]  WE: struct with the parameters for the emulator
* \param   [IN]  DGTZ: struct with the parameters for the digitizers
******************************************************************************/
void EndAcquisition(ACQparams *ACQ, WEparams *WE, DGTZparams *DGTZ);

/**************************************************************************//**
* \fn      void OpenSocket(void *arg)
* \brief   Open the socket for the communication with the Control Panel
* \param   [IN]  arg: pointer to the arguments (thread)
******************************************************************************/
void OpenSocket(void *arg);

/**************************************************************************//**
* \fn      int SocketStatus(char *msg)
* \brief   Return the status of the socket
* \param   [OUT] msg: text message that indicates the status
* \return  message code (0=success)
******************************************************************************/
int SocketStatus(char *msg);

/**************************************************************************//**
* \fn      int SocketComm(char *Data)
* \brief   Receive a data packet from the socket
* \param   [OUT] Data: data received (array of bytes)
* \return  number of bytes received
******************************************************************************/
int SocketComm(char *Data);

/**************************************************************************//**
* \fn      int RunTimeCommand(ACQparams *ACQ, WEparams *WE, DGTZparams *DGTZ)
* \brief   Read a command from the socket (or stdin) and execute it
* \param   [IN]  ACQ: struct with the parameters for the acquisition
* \param   [IN]  WE: struct with the parameters for the emulator
* \param   [IN]  DGTZ: struct with the parameters for the digitizers
* \return  Runtime command
******************************************************************************/
int RunTimeCommand(ACQparams *ACQ, WEparams *WE, DGTZparams *DGTZ);

/**************************************************************************//**
* \fn      int DigitizerMC(DGTZparams *DGTZ)
* \brief   Digitizer Manual Controller (register read/write operations)
* \param   [IN]  DGTZ: struct with the parameters for the digitizers
* \return  0: success, -1: error
******************************************************************************/
int DigitizerMC(DGTZparams *DGTZ);

#endif