/******************************************************************************
* 
* CAEN SpA - Front End Division
* Via Vetraia, 11 - 55049 - Viareggio ITALY
* +390594388398 - www.caen.it
*
***************************************************************************//**
* \note TERMS OF USE:
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the Free Software
* Foundation. This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. The user relies on the 
* software, documentation and results solely at his own risk.
*
* \file     whplot.h
* \brief    CAEN Front End - Waveform and Histogram plotter
* \author   Carlo Tintori (c.tintori@caen.it)
* \version  0.1
* \date     2009/03/05
*
* whplot is a library that allows the DPPrunner to plot waveforms and 
* histograms. It saves the data read from the digitizer (or from the 
* emulator) into a text file and send the plotiing commands to gnuplot through
* a pipe. Thus, whplot is just a simple server that transfer the data and 
* the commands from DPPrunner to gnuplot, which is the actual plotting tool.
******************************************************************************/

#ifndef __WHPLOT_H
#define __WHPLOT_H

//#include "../include/DPP.h"


/**************************************************************************//**
* \fn      FILE *OpenPlotter(void)
* \brief   Open the plotter (i.e. open gnuplot with a pipe)
* \return  file pointer (pipe); return NULL in case of error
******************************************************************************/
FILE *OpenPlotter(void);

/**************************************************************************//**
* \fn      void SetPlotOptions(FILE *gnuplot, ACQparams *ACQ)
* \brief   Set the plotting options (labels, X-Y scale, etc...)
* \param   [IN]  gnuplot: gnuplot pipe
* \param   [IN]  ACQ: struct with the parameters for the acquisition
******************************************************************************/
void SetPlotOptions(FILE *gnuplot, ACQparams *ACQ);

/**************************************************************************//**
* \fn      void PlotWaveforms(FILE *gnuplot, ACQparams *ACQ, int Channel)
* \brief   Plot the waveforms
* \param   [IN]  gnuplot: gnuplot pipe
* \param   [IN]  ACQ: struct with the parameters for the acquisition
* \param   [IN]  Channel: channel to plot
******************************************************************************/
void PlotWaveforms(FILE *gnuplot, ACQparams *ACQ, int Channel);

/**************************************************************************//**
* \fn      void PlotFFT(FILE *gnuplot, ACQparams *ACQ, int Channel)
* \brief   Plot the FFT of the virtual probe 1
* \param   [IN]  gnuplot: gnuplot pipe
* \param   [IN]  ACQ: struct with the parameters for the acquisition
* \param   [IN]  Channel: channel to plot
******************************************************************************/
void PlotFFT(FILE *gnuplot, ACQparams *ACQ, int Channel);

/**************************************************************************//**
* \fn      void PlotHistograms(FILE *gnuplot, ACQparams *ACQ, int Channel)
* \brief   Plot the histogram
* \param   [IN]  gnuplot: gnuplot pipe
* \param   [IN]  ACQ: struct with the parameters for the acquisition
* \param   [IN]  Channel: channel to plot
******************************************************************************/
void PlotHistograms(FILE *gnuplot, ACQparams *ACQ, int Channel);

/**************************************************************************//**
* \fn      void ClosePlotter(FILE *gnuplot)
* \brief   Close the plotter (gnuplot pipe)
* \param   [IN]  gnuplot: gnuplot pipe
******************************************************************************/
void ClosePlotter(FILE *gnuplot);

#endif